package de.ronnyfriedland.maven.plugin;

import java.io.File;
import java.util.Date;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import javax.naming.Context;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;

import junit.framework.Assert;

import org.apache.maven.artifact.repository.MavenArtifactRepository;
import org.apache.maven.project.MavenProject;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import de.ronnyfriedland.maven.plugin.server.EmbeddedADS;
import de.ronnyfriedland.maven.plugin.util.LdapUtils;

/**
 * Unit test for {@link LdapArtifactMojo}.
 * 
 * @author Ronny Friedland
 */
public class LdapArtifactMojoTest {

    private static final String LDAP_HOST = "localhost";
    private static final Integer LDAP_PORT = 60389;
    private static final String LDAP_USER = "uid=admin,ou=system";
    private static final String LDAP_PASSWORD = "secret";
    private static final String LDAP_ROOTDN = "ou=repository,ou=maven";
    private static final String TEST_ARTIFACTID = "maven-ldap-artifact-plugin";
    private static final String TEST_GROUPID = "de.ronnyfriedland";

    private static LdapArtifactMojo mojo;
    private static EmbeddedADS server;
    private static DirContext context;

    @BeforeClass
    public static void setUp() throws Exception {
        // start embedded ldap
        server = new EmbeddedADS(new File("target/ldap-server"));
        server.startServer();
        // initialize mojo class
        mojo = new LdapArtifactMojo(LDAP_ROOTDN, LDAP_HOST, LDAP_PORT, LDAP_USER, LDAP_PASSWORD, false, null, null,
                null, null) {
            {
                project = new MavenProject();
                project.setGroupId(TEST_GROUPID);
                project.setArtifactId(TEST_ARTIFACTID);
                project.setVersion(new Date().toString());

                local = new MavenArtifactRepository();
                local.setUrl("file://junit-test/repository");
            }
        };
        // initialize directory context
        Properties env = new Properties();
        env.put(Context.INITIAL_CONTEXT_FACTORY, "com.sun.jndi.ldap.LdapCtxFactory");
        env.put(Context.PROVIDER_URL, "ldap://" + LDAP_HOST + ":" + LDAP_PORT);
        env.put(Context.SECURITY_AUTHENTICATION, "simple");
        env.put(Context.SECURITY_PRINCIPAL, LDAP_USER);
        env.put(Context.SECURITY_CREDENTIALS, LDAP_PASSWORD);
        context = new InitialDirContext(env);
    }

    @AfterClass
    public static void tearDown() throws Exception {
        // stop embedded ldap
        server.stopServer();
    }

    /**
     * Test {@link LdapArtifactMojo#execute()}
     */
    @Test
    public void testSuccessfulEntry() throws Exception {
        Map<String, Set<String>> results = LdapUtils.findMemberOfEntry(context, LDAP_ROOTDN,
                LdapAttributes.UNIQUEMEMBER.getValue(),
                String.format("(%1$s=%2$s)", LdapAttributes.CN.getValue(), TEST_GROUPID));
        Assert.assertTrue("No result should be found", results.isEmpty());
        mojo.execute();
        results = LdapUtils.findMemberOfEntry(context, LDAP_ROOTDN, LdapAttributes.UNIQUEMEMBER.getValue(),
                String.format("(%1$s=%2$s)", LdapAttributes.CN.getValue(), TEST_GROUPID));
        Assert.assertEquals("One result should be found", 1, results.size());

    }

    /**
     * Test {@link LdapArtifactMojo#execute()}
     */
    @Test
    public void testOverwriteSuccessfulEntry() throws Exception {
        for (int i = 0; i < 2; i++) {
            mojo.execute();
        }
        Map<String, Set<String>> results = LdapUtils.findMemberOfEntry(context, LDAP_ROOTDN,
                LdapAttributes.UNIQUEMEMBER.getValue(),
                String.format("(%1$s=%2$s)", LdapAttributes.CN.getValue(), TEST_GROUPID));
        Assert.assertEquals("One result should be found", 1, results.size());
    }
}
